unit FmaUti;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Utility routines header file
//
// FmaUti.h
//
// Copyright (c) 1999 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 18/Feb/2000
//
// History: 17/Jan/1999 kvitekp - Initially created for IMA project.
//          01/Feb/2000 akzhan  - Partial conversion performed.
//          17/Feb/2000 akzhan  - Initial convertion complete.
//          18/Feb/2000 akzhan  - Adopted for FMA 1.10.93, define
//                                FMADLL11092ORLOWER to use on lower libs.
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi;

/////////////////////////////////////////////////////////////////////////////
// List management declarations
/////////////////////////////////////////////////////////////////////////////

  // List entry structure
type
  P_LST_ENTRY = ^_LST_ENTRY;
  _LST_ENTRY = packed record (* le *)
    plePrev: P_LST_ENTRY;
    pleNext: P_LST_ENTRY;
  end;
  LST_ENTRY = _LST_ENTRY;
  PLST_ENTRY = P_LST_ENTRY;

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaListAdd(var pple: PLST_ENTRY; ple: PLST_ENTRY; ile: UINT): UINT; stdcall;
function FmaListDel(var pple: PLST_ENTRY; ile: UINT): PLST_ENTRY; stdcall;
function FmaListCount(ple: PLST_ENTRY): UINT; stdcall;
{$ENDIF}
  // Special list entry index values

const
  LIST_END =   -1;          // Used by FmaListAdd()

  // Map functions to common names providing appropriate casting
{
function ListAdd(var pple: PLST_ENTRY; ple: PLST_ENTRY; ile: UINT): UINT; stdcall;
function ListDel(var pple: PLST_ENTRY; ile: UINT): PLST_ENTRY; stdcall;
function ListCount(ple: PLST_ENTRY): UINT; stdcall;
}

/////////////////////////////////////////////////////////////////////////////
// String manipulation declarations
/////////////////////////////////////////////////////////////////////////////

  // Private character manipulation macros (ascii only)
function ISLOWER(ch: AnsiChar): Boolean;
function ISUPPER(ch: AnsiChar): Boolean;
function ISDIGIT(ch: AnsiChar): Boolean;
function ISALPHA(ch: AnsiChar): Boolean;
function TOLOWER(ch: AnsiChar): AnsiChar;
function TOUPPER(ch: AnsiChar): AnsiChar;

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaSkipSpaces(var ppch: PAnsiChar): AnsiChar; stdcall;
function FmaTrimSpaces(var ppch: PAnsiChar): AnsiChar; stdcall;
function FmaAllocString(var ppsz: PAnsiChar; pch: PAnsiChar; cch: UINT): PAnsiChar; stdcall;
function FmaEqPsz(psz1, psz2: PAnsiChar): BOOL; stdcall;
function FmaEqPch(pch1, pch2: PAnsiChar; cch: UINT): BOOL; stdcall;
function FmaCpyStrMax(pchTo, pchFrom: PAnsiChar; cchMax: UINT): UINT; stdcall;
function FmaGetHash(psz: PAnsiChar): ULONG; stdcall;
function FmaGetCrc32(pb: PBYTE; cb: ULONG; crc: ULONG; bLowerCase: BOOL): ULONG; stdcall;
{$ENDIF}
//function FmaGetCrc32v(var v; cb: ULONG; crc: ULONG; bLowerCase: BOOL): ULONG; stdcall;

  // Map functions to common names
{
function SkipSpaces(var ppch: PAnsiChar): AnsiChar; stdcall;
function TrimSpaces(var ppch: PAnsiChar): AnsiChar; stdcall;
function AllocString(var ppsz: PAnsiChar; pch: PAnsiChar; cch: UINT): PAnsiChar; stdcall;
function EqPsz(psz1, psz2: PAnsiChar): BOOL; stdcall;
function EqPch(pch1, pch2: PAnsiChar; cch: UINT): BOOL; stdcall;
function CpyStrMax(pchTo, pchFrom: PAnsiChar; cchMax: UINT): UINT; stdcall;
function GetHash(psz: PAnsiChar): ULONG; stdcall;
function GetCrc32(pb: PBYTE; cb: ULONG; crc: ULONG; bLowerCase: BOOL): ULONG; stdcall;
}

/////////////////////////////////////////////////////////////////////////////
// Linked string list management declarations
/////////////////////////////////////////////////////////////////////////////

  // Linked string list entry structure

type
  PLSZ = ^LSZ;
  LSZ = packed record (* lsz *)
	  plszPrev: PLSZ;             // Pointer to the prev entry in list
	  plszNext: PLSZ;             // Pointer to the next entry in list
	  hash: ULONG;                // String hash value
//  pv: Pointer;                // Custom pointer
//  id: UINT;                   // Custom id
	  ach: array[0..0] of AnsiChar;                     // String (asciiz)
  end;

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaAddLsz(var pplsz: PLSZ; pch: PAnsiChar; cch: UINT; ilsz: UINT): PLSZ; stdcall;
function FmaGetLsz(plsz: PLSZ; psz: PAnsiChar): PLSZ; stdcall;
function FmaDelLsz(var pplsz: PLSZ; ilsz: UINT): BOOL; stdcall;
procedure FmaDelLszList(var pplsz: PLSZ); stdcall;
{$ENDIF}
  // Map functions to common names
{
function AddLsz(var pplsz: PLSZ; pch: PAnsiChar; cch: UINT; ilsz: UINT): PLSZ; stdcall;
function GetLsz(plsz: PLSZ; psz: PAnsiChar): PLSZ; stdcall;
function DelLsz(var pplsz: PLSZ; ilsz: UINT): BOOL; stdcall;
procedure DelLszList(var pplsz: PLSZ); stdcall;
}

/////////////////////////////////////////////////////////////////////////////
// Dynamic string management declarations
/////////////////////////////////////////////////////////////////////////////

  // Dynamic string structure

type
  DSZ = packed record  (* dsz *)
  	cch: UINT;                        // Max string length (not incl null)
  	ich: UINT;                        // Current string length
  	psz: PAnsiChar;                   // String pointer (asciiz)
  end;
  PDSZ = ^DSZ;

  // Dynamic string definitions

const
  DSZ_INIT: DSZ = (cch: 0; ich:0; psz: nil); // Dynamic string initializer
  DSZ_GROW      = 127;                       // Allocation grow granularity
  DSZ_GROWTWEAK = 8;                         // Allocation grow tweak factor

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaAddDsz(pdsz: PDSZ; pch: PAnsiChar; cch: UINT): BOOL; stdcall;
//  FMAAPIV_(BOOL) FmaAddDszFormat(PDSZ pdsz, PSZ pszFormat, ...);
//function FmaAddDszFormatV(pdsz: PDSZ; pszFormat: PAnsiChar; parg: Tva_list): BOOL; stdcall;
function FmaSetDsz(pdsz: PDSZ; pch: PAnsiChar; cch: UINT): BOOL; stdcall;
//  FMAAPIV_(BOOL) FmaSetDszFormat(PDSZ pdsz, PSZ pszFormat, ...);
//function FmaSetDszFormatV(pdsz: PDSZ; pszFormat: PAnsiChar; parg: Tva_list): BOOL; stdcall;
function FmaFixDsz(pdsz: PDSZ): BOOL; stdcall;
procedure FmaDelDsz(pdsz: PDSZ); stdcall;

/////////////////////////////////////////////////////////////////////////////
// Network address routines declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes

function FmaScanNetAddr(var pnetAddr: NETADDR; psz: PAnsiChar): PAnsiChar; stdcall;
function FmaFormatNetAddr(pch: PAnsiChar; cchMax: UINT; const pnetAddr: NETADDR;
							  pszDomain: PAnsiChar): UINT; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Date and time routines declarations
/////////////////////////////////////////////////////////////////////////////

  // FmaFormatDttm() flags

const
  FDTTM_DATEONLY            = $0001;        // Format date only, no time
  FDTTM_TIMEONLY            = $0002;        // Format time only, no date
  FDTTM_TIMEFIRST           = $0004;        // Format time before date
  FDTTM_TIMESECONDS         = $0008;        // Format time with seconds
  FDTTM_MONTHNUMBER         = $0010;        // Format month as number
  FDTTM_4DIGITYEAR          = $0020;        // Format full 4 digit year

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaFormatDttm(pchDttm:PAnsiChar;cchMax:UINT; dttm:PDTTM; fl:ULONG;
						 chDateSep:Char;chTimeSep:char;
						 pszDateTimeSep:PAnsiChar):UINT;stdcall;
{$ENDIF}
implementation

function ISLOWER(ch: AnsiChar): Boolean;
const
  Lowers = ['a'..'z'];
begin
  Result := ch in Lowers;
end;

function ISUPPER(ch: AnsiChar): Boolean;
const
  Uppers = ['A'..'Z'];
begin
  Result := ch in Uppers;
end;

function ISDIGIT(ch: AnsiChar): Boolean;
const
  Digits = ['0', '1'..'9'];
begin
  Result := ch in Digits;
end;

function ISALPHA(ch: AnsiChar): Boolean;
const
  Alphas = ['A'..'Z', 'a'..'z'];
begin
  Result := ch in Alphas;
end;

function TOLOWER(ch: AnsiChar): AnsiChar;
const
  Diff = Ord('a') - Ord('A');
begin
  if ISUPPER(ch) then Result := Chr( Ord(ch) + Diff ) else Result := ch;
end;

function TOUPPER(ch: AnsiChar): AnsiChar;
const
  Diff = Ord('a') - Ord('A');
begin
  if ISLOWER(ch) then Result := Chr( Ord(ch) - Diff ) else Result := ch;
end;
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaListAdd(var pple: PLST_ENTRY; ple: PLST_ENTRY; ile: UINT): UINT;
  external FmaDll name 'FmaListAdd';
function FmaListDel(var pple: PLST_ENTRY; ile: UINT): PLST_ENTRY;
  external FmaDll name 'FmaListDel';
function FmaListCount(ple: PLST_ENTRY): UINT;
  external FmaDll name 'FmaListCount';

function FmaSkipSpaces(var ppch: PAnsiChar): AnsiChar;
  external FmaDll name 'FmaSkipSpaces';
function FmaTrimSpaces(var ppch: PAnsiChar): AnsiChar;
  external FmaDll name 'FmaTrimSpaces';
function FmaAllocString(var ppsz: PAnsiChar; pch: PAnsiChar; cch: UINT): PAnsiChar;
  external FmaDll name 'FmaAllocString';
function FmaEqPsz(psz1, psz2: PAnsiChar): BOOL;
  external FmaDll name 'FmaEqPsz';
function FmaEqPch(pch1, pch2: PAnsiChar; cch: UINT): BOOL;
  external FmaDll name 'FmaEqPch';
function FmaCpyStrMax(pchTo, pchFrom: PAnsiChar; cchMax: UINT): UINT;
  external FmaDll name 'FmaCpyStrMax';
function FmaGetHash(psz: PAnsiChar): ULONG;
  external FmaDll name 'FmaGethash';
function FmaGetCrc32(pb: PBYTE; cb: ULONG; crc: ULONG; bLowerCase: BOOL): ULONG;
  external FmaDll name 'FmaGetCrc32';
//function FmaGetCrc32v(var v; cb: ULONG; crc: ULONG; bLowerCase: BOOL): ULONG;
//  external FmaDll name 'FmaGetCrc32';

function FmaAddLsz(var pplsz: PLSZ; pch: PAnsiChar; cch: UINT; ilsz: UINT): PLSZ;
  external FmaDll name 'FmaAddLsz';
function FmaGetLsz(plsz: PLSZ; psz: PAnsiChar): PLSZ;
  external FmaDll name 'FmaGetLsz';
function FmaDelLsz(var pplsz: PLSZ; ilsz: UINT): BOOL;
  external FmaDll name 'FmaDelLsz';
procedure FmaDelLszList(var pplsz: PLSZ);
  external FmaDll name 'FmaDelLszList';

function FmaAddDsz(pdsz: PDSZ; pch: PAnsiChar; cch: UINT): BOOL;
  external FmaDll name 'FmaAddDsz';
function FmaAddDszFormatV(pdsz: PDSZ; pszFormat: PAnsiChar; parg: Tva_list): BOOL;
  external FmaDll name 'FmaAddDszFormatV';
function FmaSetDsz(pdsz: PDSZ; pch: PAnsiChar; cch: UINT): BOOL;
  external FmaDll name 'FmaSetDsz';
function FmaSetDszFormatV(pdsz: PDSZ; pszFormat: PAnsiChar; parg: Tva_list): BOOL;
  external FmaDll name 'FmaSetDszFormatV';
function FmaFixDsz(pdsz: PDSZ): BOOL;
  external FmaDll name 'FmaFixDsz';
procedure FmaDelDsz(pdsz: PDSZ);
  external FmaDll name 'FmaDelDsz';

function FmaScanNetAddr(var pnetAddr: NETADDR; psz: PAnsiChar): PAnsiChar;
  external FmaDll name 'FmaScanNetAddr';
function FmaFormatNetAddr(pch: PAnsiChar; cchMax: UINT; const pnetAddr: NETADDR;
							  pszDomain: PAnsiChar): UINT;
  external FmaDll{$IFDEF FMADLL11092ORLOWER} name 'FormatNetAddr'{$ENDIF};

function FmaFormatDttm(pchDttm:PAnsiChar;cchMax:UINT; dttm:PDTTM; fl:ULONG;
						 chDateSep:Char;chTimeSep:char;
						 pszDateTimeSep:PAnsiChar):UINT; 
  external FmaDll name 'FmaFormatDttm';
{$ENDIF}
end.
