unit FmxApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Express: FmxApi Utilities that maps FMA functionality to FMX
//
// FmxApi.Pas
//
// Copyright (c) 1999-2001 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 18/Feb/2000
// Updated: 15/Feb/2001
//
// History: 18/Feb/2000 akzhan  - Initially created for Delphi convertion.
//          13/Feb/2001 akzhan  - Area and Enumeration functions added.
//          14/Feb/2001 akzhan  - Additional mappings provided.
//          15/Feb/2001 akzhan  - Cosmetic update.
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi;

type
  TFmxNetAddr = TFmaNetAddr;
{$IFDEF FMA_D5}
  TMsgIdArray = array of TMsgId;
{$ELSE}
  TMsgIdArray = array [0..0] of TMsgId;
{$ENDIF}

type
  TFmxMailPropertyType = (
    fmpPAnsiChar,
    fmpPPAnsiChar,
    fmpDWORD,
    fmpBinaryData,
    fmpDTTM,
    fmpKludge
  );

  // Mapping API prototypes
function FmxMapMailPropertyTypeToFMPID(Value: TFmxMailPropertyType): ULONG;
function FmxMapFMPIDToMailPropertyType(Value: ULONG): TFmxMailPropertyType;

type
  TEnumMsgInfo = ENUMMSGINFO;

  TEnumMsgsProc = procedure(var emi: TEnumMsgInfo; pvCallbackData: Pointer; var Cancel: Boolean);
  TEnumMsgsObjProc = procedure(var emi: TEnumMsgInfo; var Cancel: Boolean) of object;

  TEnumMsgsFlag = (emfReverseOrder, emfSafeEnum);
  TEnumMsgsFlags = set of TEnumMsgsFlag;

  // Mapping API prototypes
function FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Value: TEnumMsgsFlags): ULONG;
function FmxMapFmaEnumMsgsFlagsToEnumMsgsFlags(Value: ULONG): TEnumMsgsFlags;

type
  TFmxMsgType = (
    fmmtNative,   // Msg is native fmm message
    fmmtFTS,      // Msg is from FTS-001 message base
    fmmtSquish,   // Msg is from Squish message base
    fmmtJam,      // Msg is from JAM message base
    fmmtPkt,      // Msg is from Pkt message base
    fmmtUnknown
    );

  // Mapping API prototypes
function FmxMapMsgTypeToFmaMsgType(Value: TFmxMsgType): Integer;
function FmxMapFmaMsgTypeToMsgType(Value: Integer): TFmxMsgType;

type
  TFmxMsgAttribute = (
    maLocal,    // Msg created locally
    maInTransit,    // Msg is in-transit
    maPrivate,    // Private message
    maRead,    // Read by addressee
    maSent,    // Sent to remote
    maKillSent,    // Kill when sent
    maArchiveSent,    // Archive when sent
    maHold,    // Hold for pick-up
    maCrash,    // Crash
    maImmediate,    // Send Msg now, ignore restrictions
    maDirect,    // Send directly to destination
    maGate,    // Send via gateway
    maFileRequest,    // File request
    maFileAttach,    // File(s) attached to Msg
    maTruncFile,    // Truncate file(s) when sent
    maKillFile,    // Delete file(s) when sent
    maReceiptReq,    // Return receipt requested
    maConfirmReq,    // Confirmation receipt requested
    maOrphan,    // Unknown destination
    maEncrypt,    // Msg text is encrypted
    maCompress,    // Msg text is compressed
    maEscaped,    // Msg text is seven bit ASCII
    maForcePickup,    // Force pickup
    maTypeLocal,    // Msg is for local use only (not for export)
    maTypeEcho,    // Msg is for conference distribution
    maTypeNet,    // Msg is direct network mail
    maScanned,    // Msg is scanned (squish)
    maFileUpdateReq,    // File update request
    maAudiRequest,    // Audit request
    maNoDisplay,    // Msg may not be displayed to user
    maLocked,   // Msg is locked, no editing possible
    maDeleted    // Msg is deleted
    );
 TFmxMsgAttributes = set of TFmxMsgAttribute;

  // Mapping API prototypes
function FmxMapMsgAttributesToMsgAttr(Value: TFmxMsgAttributes): DWORD;
function FmxMapMsgAttrToMsgAttributes(Value: DWORD): TFmxMsgAttributes;

type
  TFmxAreaOpenMode = (
    aomExisting,     // Open existing, fail if does not exist
    aomAlways,       // Open existing or create if does not exist
    aomCreateNew,    // Create new, fail if already exists
    aomCreateAlways  // Create new, truncate if exists
    );

  // Mapping API prototypes
function FmxMapAreaOpenModeToFmaAreaOpenMode(Value: TFmxAreaOpenMode): UINT;
function FmxMapFmaAreaOpenModeToAreaOpenMode(Value: UINT): TFmxAreaOpenMode;

type
  TFmxAreaOpenFlag = (
    oafLockArea,        // Lock area upon opening
    oafReadOnly,        // Open area in readonly mode
    oafDenyAll,         // Open area in exclusive mode
    oafNetmail,         // Area contains netmail
    oafEchomail,        // Area contains echomail
    oafHighRead,        // Maintain highest read message (Fts/Squish)
    oafKeepUMsg,        // Keep unique message id in smh.umsg (Squish)
    oafDelayTrimming,   // Delay maxmsg trimming until close area (Squish)
    oafCreateAtEof      // Create new msgs at end of base (Squish)
    );
  TFmxAreaOpenFlags = set of TFmxAreaOpenFlag;

  // Mapping API prototypes
function FmxMapAreaOpenFlagsToFmaOpenAreaFlags(Value: TFmxAreaOpenFlags): ULONG;
function FmxMapFmaOpenAreaFlagsToAreaOpenFlags(Value: ULONG): TFmxAreaOpenFlags;

function FmxOpenArea(Path: AnsiString; _Type: TFmxMsgType; OpenMode: TFmxAreaOpenMode; Flags: TFmxAreaOpenFlags; const NetAddr: TFmxNetAddr): TAreaHandle;

function FmxCloseArea(AreaHandle: TAreaHandle): Boolean;
function FmxLockArea(AreaHandle: TAreaHandle; LockArea: Boolean): Boolean;

type
  TEnumMsgsProcData = record
    CallProc: TEnumMsgsProc;
    CallbackData: Pointer;
  end;
  PEnumMsgsProcData = ^TEnumMsgsProcData;

function EnumerateMessagesToProc(var pemi: ENUMMSGINFO; pvCallbackData: Pointer): BOOL; stdcall;

type
  TEnumMsgsObjProcData = record
    CallProc: TEnumMsgsObjProc;
  end;
  PEnumMsgsObjProcData = ^TEnumMsgsObjProcData;

function EnumerateMessagesToObjProc(var pemi: ENUMMSGINFO; pvCallbackData: Pointer): BOOL; stdcall;

function FmxEnumMsgs(AreaHandle: TAreaHandle; CallProc: TEnumMsgsProc; CallbackData: Pointer; Flags: TEnumMsgsFlags): Boolean;
function FmxEnumMsgs2(AreaHandle: TAreaHandle; CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean;

type
  TFmxMsgIdConvertionFlag = (
    micExact, // Return exact message number
    micNext,  // Return next message number
    micPrev   // Return previous message number
    );
  TFmxMsgIdConvertionMode = set of TFmxMsgIdConvertionFlag;

  // Mapping API prototypes
function FmxMapMsgIdConvertionModeToFmaMsgIdConvertionMode(Value: TFmxMsgIdConvertionMode): ULONG;
function FmxMapFmaMsgIdConvertionModeToMsgIdConvertionMode(Value: ULONG): TFmxMsgIdConvertionMode;

type
  TFmxMessageOpenMode = (momCreate, momRead, momWrite, momReadWrite);

  // Mapping API prototypes
function FmxMapMessageOpenModeToFmaMessageOpenMode(Value: TFmxMessageOpenMode): UINT;
function FmxMapFmaMessageOpenModeToMessageOpenMode(Value: UINT): TFmxMessageOpenMode;

type
  TFmxMessageOpenFlag = (mofKeepMsgId);
  TFmxMessageOpenFlags = set of TFmxMessageOpenFlag;

  // Mapping API prototypes
function FmxMapMessageOpenFlagsToFmaMessageOpenFlags(Value: TFmxMessageOpenFlags): ULONG;
function FmxMapFmaMessageOpenFlagsToMessageOpenFlags(Value: ULONG): TFmxMessageOpenFlags;

implementation

uses
  SysUtils
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

function FmxMapMailPropertyTypeToFMPID(Value: TFmxMailPropertyType): ULONG;
const
  ids: array[TFmxMailPropertyType] of ULONG = (
    FMP_ASZ,    // Asciiz string
    FMP_PSZ,    // Pointer(s) to asciiz string(s)
    FMP_U32,    // Unsigned 32-bit value(s)
    FMP_BIN,    // Counted binary of cb size
    FMP_DTM,    // Date/time info (DTTM format)
    FMP_KLU     // FNT kludge property
    );
begin
  Result := ids[Value];
end;

function FmxMapFMPIDToMailPropertyType(Value: ULONG): TFmxMailPropertyType;
const
  ids: array[TFmxMailPropertyType] of ULONG = (
    FMP_ASZ,    // Asciiz string
    FMP_PSZ,    // Pointer(s) to asciiz string(s)
    FMP_U32,    // Unsigned 32-bit value(s)
    FMP_BIN,    // Counted binary of cb size
    FMP_DTM,    // Date/time info (DTTM format)
    FMP_KLU     // FNT kludge property
    );
var
  i: TFmxMailPropertyType;
begin
  Result := fmpPAnsiChar;
  Value := Value and FMP_FLGMASK;
  for i := High(TFmxMailPropertyType) downto Low(TFmxMailPropertyType) do
  begin
    if (Value and ids[i]) = ids[i] then
    begin
      Result := i;
      Break;
    end;
  end;
end;

function FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Value: TEnumMsgsFlags): ULONG;
begin
  Result := 0;
  if emfReverseOrder in Value then
  begin
    Result := Result or FENUMMSGS_REVERSE;
  end;
  if emfSafeEnum in Value then
  begin
    Result := Result or FENUMMSGS_SAFEENUM;
  end;
end;

function FmxMapFmaEnumMsgsFlagsToEnumMsgsFlags(Value: ULONG): TEnumMsgsFlags;
begin
  Result := [];
  if (Value and FENUMMSGS_REVERSE) = FENUMMSGS_REVERSE then
  begin
    Include(Result, emfReverseOrder);
  end;
  if (Value and FENUMMSGS_SAFEENUM) = FENUMMSGS_SAFEENUM then
  begin
    Include(Result, emfReverseOrder);
  end;
end;

function EnumerateMessagesToProc(var pemi: ENUMMSGINFO; pvCallbackData: Pointer): BOOL;
var
  Cancel: Boolean;
  pData: PEnumMsgsProcData;
begin
  Cancel := False;
  pData := PEnumMsgsProcData(pvCallbackData);
  pData^.CallProc(pemi, pData^.CallbackData, Cancel);
  Result := not Cancel;
end;

function FmxEnumMsgs(AreaHandle: TAreaHandle; CallProc: TEnumMsgsProc; CallbackData: Pointer; Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsProcData;
begin
  Data.CallProc := CallProc;
  Data.CallbackData := CallbackData;
  Result :=
    FmaEnumMsgs(AreaHandle, EnumerateMessagesToProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function EnumerateMessagesToObjProc(var pemi: ENUMMSGINFO; pvCallbackData: Pointer): BOOL;
var
  Cancel: Boolean;
  pData: PEnumMsgsObjProcData;
begin
  Cancel := False;
  pData := PEnumMsgsObjProcData(pvCallbackData);
  pData^.CallProc(pemi, Cancel);
  Result := not Cancel;
end;

function FmxEnumMsgs2(AreaHandle: TAreaHandle; CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsObjProcData;
begin
  Data.CallProc := CallProc;
  Result :=
    FmaEnumMsgs(AreaHandle, EnumerateMessagesToObjProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function FmxMapMsgTypeToFmaMsgType(Value: TFmxMsgType): Integer;
begin
  Result := Ord(Value) - Ord(Low(TFmxMsgType)) + FMMT_NATIVE;
end;

function FmxMapFmaMsgTypeToMsgType(Value: Integer): TFmxMsgType;
const
  types: array[0..4] of TFmxMsgType = (
    fmmtNative,   // Msg is native fmm message
    fmmtFTS,      // Msg is from FTS-001 message base
    fmmtSquish,   // Msg is from Squish message base
    fmmtJam,      // Msg is from JAM message base
    fmmtPkt       // Msg is from Pkt message base
    );
begin
  if (Value >= Low(types)) and (Value <= High(types)) then
  begin
    Result := types[Value];
  end
  else
  begin
    Result := fmmtUnknown;
  end;
end;

function FmxMapMsgAttributesToMsgAttr(Value: TFmxMsgAttributes): DWORD;
var
  i: TFmxMsgAttribute;
  bit: DWORD;
begin
  Result := 0;
  bit := 1;
  for i := Low(TFmxMsgAttribute) to High(TFmxMsgAttribute) do
  begin
    if i in Value then
    begin
      Result := Result or bit;
    end;
    bit := bit shl 1;
  end;
end;

function FmxMapMsgAttrToMsgAttributes(Value: DWORD): TFmxMsgAttributes;
var
  i: TFmxMsgAttribute;
  bit: DWORD;
begin
  Result := [];
  bit := 1;
  for i := Low(TFmxMsgAttribute) to High(TFmxMsgAttribute) do
  begin
    if (Value and bit) <> 0 then
    begin
      Include(Result, i);
    end;
    bit := bit shl 1;
  end;
end;

function FmxMapAreaOpenModeToFmaAreaOpenMode(Value: TFmxAreaOpenMode): UINT;
begin
  Result := Ord(Value) - Ord(Low(TFmxAreaOpenMode)) + OPENAREA_EXISTING;
end;

function FmxMapFmaAreaOpenModeToAreaOpenMode(Value: UINT): TFmxAreaOpenMode;
begin
  Result := TFmxAreaOpenMode(Value - OPENAREA_EXISTING);
end;

function FmxMapAreaOpenFlagsToFmaOpenAreaFlags(Value: TFmxAreaOpenFlags): ULONG;
begin
  Result := 0;
  if oafLockArea in Value then
  begin
    Result := Result or FOA_LOCKAREA;
  end;
  if oafReadOnly in Value then
  begin
    Result := Result or FOA_READONLY;
  end;
  if oafDenyAll in Value then
  begin
    Result := Result or FOA_DENYALL;
  end;
  if oafNetmail in Value then
  begin
    Result := Result or FOA_NETMAIL;
  end;
  if oafEchomail in Value then
  begin
    Result := Result or FOA_ECHOMAIL;
  end;
  if oafHighRead in Value then
  begin
    Result := Result or FOA_HIGHREAD;
  end;
  if oafKeepUMsg in Value then
  begin
    Result := Result or FOA_KEEPUMSG;
  end;
  if oafDelayTrimming in Value then
  begin
    Result := Result or FOA_DELAYTRIMMING;
  end;
  if oafCreateAtEof in Value then
  begin
    Result := Result or FOA_CREATEATEOF;
  end;
end;

function FmxMapFmaOpenAreaFlagsToAreaOpenFlags(Value: ULONG): TFmxAreaOpenFlags;
begin
  Result := [];
  if (Value and FOA_LOCKAREA) = FOA_LOCKAREA then
  begin
    Include(Result, oafLockArea);
  end;
  if (Value and FOA_READONLY) = FOA_READONLY then
  begin
    Include(Result, oafReadOnly);
  end;
  if (Value and FOA_DENYALL) = FOA_DENYALL then
  begin
    Include(Result, oafDenyAll);
  end;
  if (Value and FOA_NETMAIL) = FOA_NETMAIL then
  begin
    Include(Result, oafNetmail);
  end;
  if (Value and FOA_ECHOMAIL) = FOA_ECHOMAIL then
  begin
    Include(Result, oafEchomail);
  end;
  if (Value and FOA_HIGHREAD) = FOA_HIGHREAD then
  begin
    Include(Result, oafHighRead);
  end;
  if (Value and FOA_KEEPUMSG) = FOA_KEEPUMSG then
  begin
    Include(Result, oafKeepUMsg);
  end;
  if (Value and FOA_DELAYTRIMMING) = FOA_DELAYTRIMMING then
  begin
    Include(Result, oafDelayTrimming);
  end;
  if (Value and FOA_CREATEATEOF) = FOA_CREATEATEOF then
  begin
    Include(Result, oafCreateAtEof);
  end;
end;

function FmxMapMsgIdConvertionModeToFmaMsgIdConvertionMode(Value: TFmxMsgIdConvertionMode): ULONG;
begin
  Result := 0;
  if micExact in Value then
  begin
    Result := Result or UMSG_EXACT;
  end;
  if micNext in Value then
  begin
    Result := Result or UMSG_NEXT;
  end;
  if micPrev in Value then
  begin
    Result := Result or UMSG_PREV;
  end;
end;

function FmxMapFmaMsgIdConvertionModeToMsgIdConvertionMode(Value: ULONG): TFmxMsgIdConvertionMode;
begin
  Result := [];
  if (Value and UMSG_EXACT) = UMSG_EXACT then
  begin
    Include(Result, micExact);
  end;
  if (Value and UMSG_NEXT) = UMSG_NEXT then
  begin
    Include(Result, micNext);
  end;
  if (Value and UMSG_PREV) = UMSG_PREV then
  begin
    Include(Result, micPrev);
  end;
end;

function FmxOpenArea(Path: AnsiString; _Type: TFmxMsgType; OpenMode: TFmxAreaOpenMode; Flags: TFmxAreaOpenFlags; const NetAddr: TFmxNetAddr): TAreaHandle;
var
  TempAddr: TFmaNetAddr;
begin
  TempAddr := NetAddr;
  Result :=
    FmaOpenArea(
      PAnsiChar(Path),
      FmxMapMsgTypeToFmaMsgType(_Type),
      FmxMapAreaOpenModeToFmaAreaOpenMode(OpenMode),
      FmxMapAreaOpenFlagsToFmaOpenAreaFlags(Flags),
      TempAddr
      );
end;

function FmxCloseArea(AreaHandle: TAreaHandle): Boolean;
begin
  Result := FmaCloseArea(AreaHandle);
end;

function FmxLockArea(AreaHandle: TAreaHandle; LockArea: Boolean): Boolean;
begin
  Result := FmaLockArea(AreaHandle, LockArea);
end;

function FmxMapMessageOpenModeToFmaMessageOpenMode(Value: TFmxMessageOpenMode): UINT;
begin
  Result := Ord(Value) - Ord(momCreate) + OPENMSG_CREATE;
end;

function FmxMapFmaMessageOpenModeToMessageOpenMode(Value: UINT): TFmxMessageOpenMode;
begin
  Result := TFmxMessageOpenMode(Value - OPENMSG_CREATE + Cardinal(Ord(momCreate)));
end;

function FmxMapMessageOpenFlagsToFmaMessageOpenFlags(Value: TFmxMessageOpenFlags): ULONG;
begin
  if mofKeepMsgId in Value then
  begin
    Result := FOM_KEEPUMSG;
  end
  else
  begin
    Result := 0;
  end;
end;

function FmxMapFmaMessageOpenFlagsToMessageOpenFlags(Value: ULONG): TFmxMessageOpenFlags;
begin
  if (Value and FOM_KEEPUMSG) = FOM_KEEPUMSG then
  begin
    Result := [mofKeepMsgId];
  end
  else
  begin
    Result := [];
  end;
end;

end.
