unit FmxFts;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Express: FMX Classes that provides object APIs
//                       for FMA/FTS.
//
// FmxFts.Pas
//
// Copyright (c) 2001 by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 13/Feb/2001
// Updated: 14/Oct/2002
//
// History: 13/Feb/2001 akzhan  - Initially created for Delphi.
//          14/Feb/2001 akzhan  - Alpha 1.
//          15/Feb/2001 akzhan  - Beta (all functionality implemented).
//          14/Oct/2002 Plyako A- abstract method DoGetMessage overrided
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi, FmxApi, FmxCore;

type
  TFmxFTSAreaMessage = class(TFmxCustomAreaMessage)
  protected
    function DoClose: Boolean; override;
    function DoGetControlInfoLen(var Len: ULONG): Boolean; override;
    function DoGetTextLen(var Len: ULONG): Boolean; override;

    function DoLoad(var ppfmm: PMsgObject; fl: ULONG): Boolean; override;
    function DoSave(pfmm: PMsgObject; fl: ULONG; cchTextTotal: ULONG): Boolean; override;

    function DoRead(pmsgh: Pointer; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): Boolean; override;
    function DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar; cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): Boolean; override;
  public
    constructor Create(AreaHandle: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG); override;
  end;

type
  TFmxFTSMessageArea = class(TFmxCustomMessageArea)
  protected
    procedure DoOpen(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR); override;
    function DoClose: Boolean; override;
    function DoLock(LockArea: Boolean): Boolean; override;
    function DoGetLRInfo(var lri: LRINFO): Boolean; override;
    function DoSetLRInfo(const lri: LRINFO): Boolean; override;
    function DoValidateMessageNumber(MsgN: TMsgNumber; var MsgIndex: ULONG): Boolean; override;
    function DoCreateMsgMap(var paumsg: PUMSG; var pcumsg: ULONG): Boolean; override;
    function DoEnumMessages(CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean; override;
    function DoGetCurrentMessage: TMsgNumber; override;
    function DoSetCurrentMessage(msgn: TMsgNumber): Boolean; override;
    function DoGetFirstMessage: TMsgNumber; override;
    function DoGetHighMessage: TMsgNumber; override;
    function DoGetMessageCount: LONG; override;
    function DoDeleteMessage(msgn: TMsgNumber): Boolean; override;
    function DoMsgIdToMsgNumber(Value: TMsgId; Mode: UINT): TMsgNumber; override;
    function DoMsgNumberToMsgId(Value: TMsgNumber): TMsgId; override;
    function DoGetMessage(msgn: TMsgNumber; Mode: UINT; Flags: ULONG): TFmxCustomAreaMessage; override;
  end;


implementation

uses
  FtsApi
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

{ TFmxFTSMessageArea }

function TFmxFTSMessageArea.DoClose: Boolean;
begin
  Result := FtsCloseArea(Handle);
end;

function TFmxFTSMessageArea.DoCreateMsgMap(var paumsg: PUMSG;
  var pcumsg: ULONG): Boolean;
begin
  Result := FtsCreateMsgMap(Handle, paumsg, pcumsg);
end;

function TFmxFTSMessageArea.DoDeleteMessage(msgn: TMsgNumber): Boolean;
begin
  Result := FtsDeleteMsg(Handle, msgn);
end;

function TFmxFTSMessageArea.DoEnumMessages(CallProc: TEnumMsgsObjProc;
  Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsObjProcData;
begin
  Data.CallProc := CallProc;
  Result :=
    FtsEnumMsgs(Handle, EnumerateMessagesToObjProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function TFmxFTSMessageArea.DoGetCurrentMessage: TMsgNumber;
begin
  Result := FtsGetCurMsg(Handle);
end;

function TFmxFTSMessageArea.DoGetFirstMessage: TMsgNumber;
begin
  Result := FtsGetFirstMsg(Handle);
end;

function TFmxFTSMessageArea.DoGetHighMessage: TMsgNumber;
begin
  Result := FtsGetHighMsg(Handle);
end;

function TFmxFTSMessageArea.DoGetLRInfo(var lri: LRINFO): Boolean;
begin
  Result := FtsGetLRInfo(Handle, UserId, Addr(lri));
end;

function TFmxFTSMessageArea.DoGetMessage(msgn: TMsgNumber; Mode: UINT;
  Flags: ULONG): TFmxCustomAreaMessage;
begin
  Result :=
    TFmxFTSAreaMessage.Create(Handle, Mode, msgn, Flags);
end;

function TFmxFTSMessageArea.DoGetMessageCount: LONG;
begin
  Result := FtsGetMsgCount(Handle);
end;

function TFmxFTSMessageArea.DoLock(LockArea: Boolean): Boolean;
begin
  Result := FtsLockArea(Handle, LockArea);
end;

function TFmxFTSMessageArea.DoMsgIdToMsgNumber(
  Value: TMsgId; Mode: UINT): TMsgNumber;
begin
  Result := FtsUidToMsgn(Handle, Value, Mode);
end;

function TFmxFTSMessageArea.DoMsgNumberToMsgId(
  Value: TMsgNumber): TMsgId;
begin
  Result := FtsMsgnToUid(Handle, Value);
end;

procedure TFmxFTSMessageArea.DoOpen(pszPath: PAnsiChar; fOpenMode: UINT;
  fl: ULONG; var pnetAddr: NETADDR);
begin
  SetHandle(
    FtsOpenArea(
      pszPath,
      fOpenMode,
      fl,
      pnetAddr
      )
    );
end;

function TFmxFTSMessageArea.DoSetCurrentMessage(
  msgn: TMsgNumber): Boolean;
begin
  Result := FtsSetCurMsg(Handle, msgn);
end;

function TFmxFTSMessageArea.DoSetLRInfo(const lri: LRINFO): Boolean;
begin
  Result := FtsSetLRInfo(Handle, UserId, Addr(lri), PAnsiChar(UserName));
end;

function TFmxFTSMessageArea.DoValidateMessageNumber(MsgN: TMsgNumber;
  var MsgIndex: ULONG): Boolean;
begin
  Result := FtsValidateMsg(Handle, MsgN, Addr(MsgIndex));
end;

{ TFmxFTSAreaMessage }

constructor TFmxFTSAreaMessage.Create(AreaHandle: TAreaHandle;
  mode: UINT; msgn: TMsgNumber; fl: ULONG);
begin
  inherited CreateOn(
    FtsOpenMsg(AreaHandle, mode, msgn, fl)
    );
end;

function TFmxFTSAreaMessage.DoClose: Boolean;
begin
  Result := FtsCloseMsg(Handle);
end;

function TFmxFTSAreaMessage.DoGetControlInfoLen(
  var Len: ULONG): Boolean;
begin
  Result := FtsGetMsgCtrlLen(Handle, Len);
end;

function TFmxFTSAreaMessage.DoGetTextLen(var Len: ULONG): Boolean;
begin
  Result := FtsGetMsgTextLen(Handle, Len);
end;

function TFmxFTSAreaMessage.DoLoad(var ppfmm: PMsgObject;
  fl: ULONG): Boolean;
begin
  Result := FtsLoadMsg(
    Handle,
    ppfmm,
    fl
    );
end;

function TFmxFTSAreaMessage.DoRead(pmsgh: Pointer;
  var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar;
  pcchText, poffText: PULONG): Boolean;
begin
  Result := FtsReadMsg(
    Handle,
    pmsgh,
    ppchCtrl,
    pcchCtrl,
    ppchText,
    pcchText,
    poffText
    );
end;

function TFmxFTSAreaMessage.DoSave(pfmm: PMsgObject; fl,
  cchTextTotal: ULONG): Boolean;
begin
  Result := FtsSaveMsg(
    Handle,
    pfmm,
    fl,
    cchTextTotal
    );
end;

function TFmxFTSAreaMessage.DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar;
  cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG;
  cchTextTotal: ULONG): Boolean;
begin
  Result := FtsWriteMsg(
    Handle,
    pmsgh,
    pchCtrl,
    cchCtrl,
    pchText,
    cchText,
    poffText,
    cchTextTotal
    );
end;

end.
