unit FmxPkt;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Express: FMX Classes that provides object APIs
//                       for FMA/PKT.
//
// FmxPkt.Pas
//
// Copyright (c) 2001 by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 13/Feb/2001
// Updated: 14/Oct/2002
//
// History: 13/Feb/2001 akzhan  - Initially created for Delphi.
//          14/Feb/2001 akzhan  - Alpha 1.
//          15/Feb/2001 akzhan  - Beta (all functionality implemented).
//          14/Oct/2002 Plyako A- abstract method DoGetMessage overrided
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi, FmxApi, FmxCore;

type
  TFmxPKTAreaMessage = class(TFmxCustomAreaMessage)
  protected
    function DoClose: Boolean; override;
    function DoGetControlInfoLen(var Len: ULONG): Boolean; override;
    function DoGetTextLen(var Len: ULONG): Boolean; override;

    function DoLoad(var ppfmm: PMsgObject; fl: ULONG): Boolean; override;
    function DoSave(pfmm: PMsgObject; fl: ULONG; cchTextTotal: ULONG): Boolean; override;

    function DoRead(pmsgh: Pointer; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): Boolean; override;
    function DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar; cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): Boolean; override;
  public
    constructor Create(AreaHandle: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG); override;
  end;

type
  TFmxPKTMessageArea = class(TFmxCustomMessageArea)
  protected
    procedure DoOpen(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR); override;
    function DoClose: Boolean; override;
    function DoLock(LockArea: Boolean): Boolean; override;
    function DoGetLRInfo(var lri: LRINFO): Boolean; override;
    function DoSetLRInfo(const lri: LRINFO): Boolean; override;
    function DoValidateMessageNumber(MsgN: TMsgNumber; var MsgIndex: ULONG): Boolean; override;
    function DoCreateMsgMap(var paumsg: PUMSG; var pcumsg: ULONG): Boolean; override;
    function DoEnumMessages(CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean; override;
    function DoGetCurrentMessage: TMsgNumber; override;
    function DoSetCurrentMessage(msgn: TMsgNumber): Boolean; override;
    function DoGetFirstMessage: TMsgNumber; override;
    function DoGetHighMessage: TMsgNumber; override;
    function DoGetMessageCount: LONG; override;
    function DoDeleteMessage(msgn: TMsgNumber): Boolean; override;
    function DoMsgIdToMsgNumber(Value: TMsgId; Mode: UINT): TMsgNumber; override;
    function DoMsgNumberToMsgId(Value: TMsgNumber): TMsgId; override;
    function DoGetMessage(msgn: TMsgNumber; Mode: UINT; Flags: ULONG): TFmxCustomAreaMessage; override;
  end;


implementation

uses
  PktApi
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

{ TFmxPKTMessageArea }

function TFmxPKTMessageArea.DoClose: Boolean;
begin
  Result := PktCloseArea(Handle);
end;

function TFmxPKTMessageArea.DoCreateMsgMap(var paumsg: PUMSG;
  var pcumsg: ULONG): Boolean;
begin
  Result := PktCreateMsgMap(Handle, paumsg, pcumsg);
end;

function TFmxPKTMessageArea.DoDeleteMessage(msgn: TMsgNumber): Boolean;
begin
  Result := PktDeleteMsg(Handle, msgn);
end;

function TFmxPKTMessageArea.DoEnumMessages(CallProc: TEnumMsgsObjProc;
  Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsObjProcData;
begin
  Data.CallProc := CallProc;
  Result :=
    PktEnumMsgs(Handle, EnumerateMessagesToObjProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function TFmxPKTMessageArea.DoGetCurrentMessage: TMsgNumber;
begin
  Result := PktGetCurMsg(Handle);
end;

function TFmxPKTMessageArea.DoGetFirstMessage: TMsgNumber;
begin
  Result := PktGetFirstMsg(Handle);
end;

function TFmxPKTMessageArea.DoGetHighMessage: TMsgNumber;
begin
  Result := PktGetHighMsg(Handle);
end;

function TFmxPKTMessageArea.DoGetLRInfo(var lri: LRINFO): Boolean;
begin
  Result := PktGetLRInfo(Handle, UserId, Addr(lri));
end;

function TFmxPKTMessageArea.DoGetMessage(msgn: TMsgNumber; Mode: UINT;
  Flags: ULONG): TFmxCustomAreaMessage;
begin
  Result :=
    TFmxPktAreaMessage.Create(Handle, Mode, msgn, Flags);
end;

function TFmxPKTMessageArea.DoGetMessageCount: LONG;
begin
  Result := PktGetMsgCount(Handle);
end;

function TFmxPKTMessageArea.DoLock(LockArea: Boolean): Boolean;
begin
  Result := PktLockArea(Handle, LockArea);
end;

function TFmxPKTMessageArea.DoMsgIdToMsgNumber(
  Value: TMsgId; Mode: UINT): TMsgNumber;
begin
  Result := PktUidToMsgn(Handle, Value, Mode);
end;

function TFmxPKTMessageArea.DoMsgNumberToMsgId(
  Value: TMsgNumber): TMsgId;
begin
  Result := PktMsgnToUid(Handle, Value);
end;

procedure TFmxPKTMessageArea.DoOpen(pszPath: PAnsiChar; fOpenMode: UINT;
  fl: ULONG; var pnetAddr: NETADDR);
begin
  SetHandle(
    PktOpenArea(
      pszPath,
      fOpenMode,
      fl,
      pnetAddr
      )
    );
end;

function TFmxPKTMessageArea.DoSetCurrentMessage(
  msgn: TMsgNumber): Boolean;
begin
  Result := PktSetCurMsg(Handle, msgn);
end;

function TFmxPKTMessageArea.DoSetLRInfo(const lri: LRINFO): Boolean;
begin
  Result := PktSetLRInfo(Handle, UserId, Addr(lri), PAnsiChar(UserName));
end;

function TFmxPKTMessageArea.DoValidateMessageNumber(MsgN: TMsgNumber;
  var MsgIndex: ULONG): Boolean;
begin
  Result := PktValidateMsg(Handle, MsgN, Addr(MsgIndex));
end;

{ TFmxPKTAreaMessage }

constructor TFmxPKTAreaMessage.Create(AreaHandle: TAreaHandle;
  mode: UINT; msgn: TMsgNumber; fl: ULONG);
begin
  inherited CreateOn(
    PktOpenMsg(AreaHandle, mode, msgn, fl)
    );
end;

function TFmxPKTAreaMessage.DoClose: Boolean;
begin
  Result := PktCloseMsg(Handle);
end;

function TFmxPKTAreaMessage.DoGetControlInfoLen(
  var Len: ULONG): Boolean;
begin
  Result := PktGetMsgCtrlLen(Handle, Len);
end;

function TFmxPKTAreaMessage.DoGetTextLen(var Len: ULONG): Boolean;
begin
  Result := PktGetMsgTextLen(Handle, Len);
end;

function TFmxPKTAreaMessage.DoLoad(var ppfmm: PMsgObject;
  fl: ULONG): Boolean;
begin
  Result := PktLoadMsg(
    Handle,
    ppfmm,
    fl
    );
end;

function TFmxPKTAreaMessage.DoRead(pmsgh: Pointer;
  var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar;
  pcchText, poffText: PULONG): Boolean;
begin
  Result := PktReadMsg(
    Handle,
    pmsgh,
    ppchCtrl,
    pcchCtrl,
    ppchText,
    pcchText,
    poffText
    );
end;

function TFmxPKTAreaMessage.DoSave(pfmm: PMsgObject; fl,
  cchTextTotal: ULONG): Boolean;
begin
  Result := PktSaveMsg(
    Handle,
    pfmm,
    fl,
    cchTextTotal
    );
end;

function TFmxPKTAreaMessage.DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar;
  cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG;
  cchTextTotal: ULONG): Boolean;
begin
  Result := PktWriteMsg(
    Handle,
    pmsgh,
    pchCtrl,
    cchCtrl,
    pchText,
    cchText,
    poffText,
    cchTextTotal
    );
end;

end.
