unit FmxSqu;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Express: FMX Classes that provides object APIs
//                       for FMA/Squish.
//
// FmxSqu.Pas
//
// Copyright (c) 2001 by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 13/Feb/2001
// Updated: 14/Oct/2002
//
// History: 13/Feb/2001 akzhan  - Initially created for Delphi.
//          14/Feb/2001 akzhan  - Alpha 1.
//          15/Feb/2001 akzhan  - Beta (all functionality implemented).
//          14/Oct/2002 Plyako A- abstract method DoGetMessage overrided
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi, FmxApi, FmxCore;

type
  TFmxSquishAreaMessage = class(TFmxCustomAreaMessage)
  protected
    function DoClose: Boolean; override;
    function DoGetControlInfoLen(var Len: ULONG): Boolean; override;
    function DoGetTextLen(var Len: ULONG): Boolean; override;

    function DoLoad(var ppfmm: PMsgObject; fl: ULONG): Boolean; override;
    function DoSave(pfmm: PMsgObject; fl: ULONG; cchTextTotal: ULONG): Boolean; override;

    function DoRead(pmsgh: Pointer; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): Boolean; override;
    function DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar; cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): Boolean; override;
  public
    constructor Create(AreaHandle: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG); override;
  end;

type
  TFmxSquishMessageArea = class(TFmxCustomMessageArea)
  protected
    procedure DoOpen(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR); override;
    function DoClose: Boolean; override;
    function DoLock(LockArea: Boolean): Boolean; override;
    function DoGetLRInfo(var lri: LRINFO): Boolean; override;
    function DoSetLRInfo(const lri: LRINFO): Boolean; override;
    function DoValidateMessageNumber(MsgN: TMsgNumber; var MsgIndex: ULONG): Boolean; override;
    function DoCreateMsgMap(var paumsg: PUMSG; var pcumsg: ULONG): Boolean; override;
    function DoEnumMessages(CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean; override;
    function DoGetCurrentMessage: TMsgNumber; override;
    function DoSetCurrentMessage(msgn: TMsgNumber): Boolean; override;
    function DoGetFirstMessage: TMsgNumber; override;
    function DoGetHighMessage: TMsgNumber; override;
    function DoGetMessageCount: LONG; override;
    function DoDeleteMessage(msgn: TMsgNumber): Boolean; override;
    function DoMsgIdToMsgNumber(Value: TMsgId; Mode: UINT): TMsgNumber; override;
    function DoMsgNumberToMsgId(Value: TMsgNumber): TMsgId; override;
    function DoGetMessage(msgn: TMsgNumber; Mode: UINT; Flags: ULONG): TFmxCustomAreaMessage; override;
  end;


implementation

uses
  SquApi
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

{ TFmxSquishMessageArea }

function TFmxSquishMessageArea.DoClose: Boolean;
begin
  Result := SquCloseArea(Handle);
end;

function TFmxSquishMessageArea.DoCreateMsgMap(var paumsg: PUMSG;
  var pcumsg: ULONG): Boolean;
begin
  Result := SquCreateMsgMap(Handle, paumsg, pcumsg);
end;

function TFmxSquishMessageArea.DoDeleteMessage(msgn: TMsgNumber): Boolean;
begin
  Result := SquDeleteMsg(Handle, msgn);
end;

function TFmxSquishMessageArea.DoEnumMessages(CallProc: TEnumMsgsObjProc;
  Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsObjProcData;
begin
  Data.CallProc := CallProc;
  Result :=
    SquEnumMsgs(Handle, EnumerateMessagesToObjProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function TFmxSquishMessageArea.DoGetCurrentMessage: TMsgNumber;
begin
  Result := SquGetCurMsg(Handle);
end;

function TFmxSquishMessageArea.DoGetFirstMessage: TMsgNumber;
begin
  Result := SquGetFirstMsg(Handle);
end;

function TFmxSquishMessageArea.DoGetHighMessage: TMsgNumber;
begin
  Result := SquGetHighMsg(Handle);
end;

function TFmxSquishMessageArea.DoGetLRInfo(var lri: LRINFO): Boolean;
begin
  Result := SquGetLRInfo(Handle, UserId, Addr(lri));
end;

function TFmxSquishMessageArea.DoGetMessage(msgn: TMsgNumber;
  Mode: UINT; Flags: ULONG): TFmxCustomAreaMessage;
begin
  Result :=
    TFmxSquishAreaMessage.Create(Handle, Mode, msgn, Flags);
end;

function TFmxSquishMessageArea.DoGetMessageCount: LONG;
begin
  Result := SquGetMsgCount(Handle);
end;

function TFmxSquishMessageArea.DoLock(LockArea: Boolean): Boolean;
begin
  Result := SquLockArea(Handle, LockArea);
end;

function TFmxSquishMessageArea.DoMsgIdToMsgNumber(
  Value: TMsgId; Mode: UINT): TMsgNumber;
begin
  Result := SquUidToMsgn(Handle, Value, Mode);
end;

function TFmxSquishMessageArea.DoMsgNumberToMsgId(
  Value: TMsgNumber): TMsgId;
begin
  Result := SquMsgnToUid(Handle, Value);
end;

procedure TFmxSquishMessageArea.DoOpen(pszPath: PAnsiChar; fOpenMode: UINT;
  fl: ULONG; var pnetAddr: NETADDR);
begin
  SetHandle(
    SquOpenArea(
      pszPath,
      fOpenMode,
      fl,
      pnetAddr
      )
    );
end;

function TFmxSquishMessageArea.DoSetCurrentMessage(
  msgn: TMsgNumber): Boolean;
begin
  Result := SquSetCurMsg(Handle, msgn);
end;

function TFmxSquishMessageArea.DoSetLRInfo(const lri: LRINFO): Boolean;
begin
  Result := SquSetLRInfo(Handle, UserId, Addr(lri), PAnsiChar(UserName));
end;

function TFmxSquishMessageArea.DoValidateMessageNumber(MsgN: TMsgNumber;
  var MsgIndex: ULONG): Boolean;
begin
  Result := SquValidateMsg(Handle, MsgN, Addr(MsgIndex));
end;

{ TFmxSquishAreaMessage }

constructor TFmxSquishAreaMessage.Create(AreaHandle: TAreaHandle;
  mode: UINT; msgn: TMsgNumber; fl: ULONG);
begin
  inherited CreateOn(
    SquOpenMsg(AreaHandle, mode, msgn, fl)
    );
end;

function TFmxSquishAreaMessage.DoClose: Boolean;
begin
  Result := SquCloseMsg(Handle);
end;

function TFmxSquishAreaMessage.DoGetControlInfoLen(
  var Len: ULONG): Boolean;
begin
  Result := SquGetMsgCtrlLen(Handle, Len);
end;

function TFmxSquishAreaMessage.DoGetTextLen(var Len: ULONG): Boolean;
begin
  Result := SquGetMsgTextLen(Handle, Len);
end;

function TFmxSquishAreaMessage.DoLoad(var ppfmm: PMsgObject;
  fl: ULONG): Boolean;
begin
  Result := SquLoadMsg(
    Handle,
    ppfmm,
    fl
    );
end;

function TFmxSquishAreaMessage.DoRead(pmsgh: Pointer;
  var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar;
  pcchText, poffText: PULONG): Boolean;
begin
  Result := SquReadMsg(
    Handle,
    pmsgh,
    ppchCtrl,
    pcchCtrl,
    ppchText,
    pcchText,
    poffText
    );
end;

function TFmxSquishAreaMessage.DoSave(pfmm: PMsgObject; fl,
  cchTextTotal: ULONG): Boolean;
begin
  Result := SquSaveMsg(
    Handle,
    pfmm,
    fl,
    cchTextTotal
    );
end;

function TFmxSquishAreaMessage.DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar;
  cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG;
  cchTextTotal: ULONG): Boolean;
begin
  Result := SquWriteMsg(
    Handle,
    pmsgh,
    pchCtrl,
    cchCtrl,
    pchText,
    cchText,
    poffText,
    cchTextTotal
    );
end;

end.
