unit FtsApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Fts api declarations module
//
// FtsApi.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 18/Feb/2000
//
// History: 27/Aug/1999 kvitekp - Initially created.
//          18/Feb/2000 akzhan  - Convertion complete.
//
/////////////////////////////////////////////////////////////////////////////
//
// Note: These declarations were derived from original
//       specifications published in FTS-0001.16 and
//       some common real life practices.
//
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: ALL STRUCTURES IN THIS MODULE ASSUME SINGLE BYTE PACKING !!!
//
/////////////////////////////////////////////////////////////////////////////
{$I FmxDef.inc}
interface

uses
  Windows, FmaDef, FmaApi;

  // Fts base file extensions

const
  FTS_MSG_FILEEXT = '.MSG'; // Message file extension
  FTS_LRD_FILEEXT = '.BBS'; // Lastread file extension

  // Fts base file names

const
  FTS_LRD_FILENAME = 'LASTREAD';  // Lastread file name

  // Fts message header structure mostly used

type
  _FMH = packed record (* fmh *)
    achBy: packed array[0..35] of AnsiChar;   // From name
    achTo: packed array[0..35] of AnsiChar;   // To name
    achRe: packed array[0..71] of AnsiChar;   // Message subject
    achDate: packed array[0..19] of AnsiChar; // ASCII date information
    timesRead: USHORT;                 // Number of times read
    destNode: USHORT;                  // Destination node number
    origNode: USHORT;                  // Origination node number
    cost: USHORT;                      // Unit cost charged to send the message
    origNet: USHORT;                   // Origination net number
    destNet: USHORT;                   // Destination net number
    dttmWritten: DTTM;                 // Message written date/time stamp
    dttmArrived: DTTM;                 // Message arrived date/time stamp
    replyTo: USHORT;                   // Msg is a reply to this msg number
    attr: USHORT;                      // Message attributes (FMA_)
    nextReply: USHORT;                 // Next message in the reply thread
  end;
  FMH = _FMH;
  PFMH = ^FMH;

  // Fts message header structure according to FTS-001.16 with
  // zone/point info in place of dttm's, nice, but rarely used

type
  _FMH2 = packed record (* fmh2 *)
    achBy: packed array[0..35] of AnsiChar;   // From name
    achTo: packed array[0..35] of AnsiChar;   // To name
    achRe: packed array[0..71] of AnsiChar;   // Message subject
    achDate: packed array[0..19] of AnsiChar; // ASCII date information
    timesRead: USHORT;                 // Number of times read
    destNode: USHORT;                  // Destination node number
    origNode: USHORT;                  // Origination node number
    cost: USHORT;                      // Unit cost charged to send the message
    origNet: USHORT;                   // Origination net number
    destNet: USHORT;                   // Destination net number
    destZone: USHORT;                  // Destination zone number
    origZone: USHORT;                  // Origination zone number
    destPoint: USHORT;                 // Destination point number
    origPoint: USHORT;                 // Origination point number
    replyTo: USHORT;                   // Msg is a reply to this msg number
    attr: USHORT;                      // Message attributes (FMA_)
    nextReply: USHORT;                 // Next message in the reply thread
  end;
  FMH2 = _FMH2;
  PFMH2 = ^FMH2;

  // Fts message attributes in fmh.attr

const
  FMA_PRVT     = $0001;    // Private                 +
  FMA_CRSH     = $0002;    // Crash                   | These
  FMA_READ     = $0004;    // Read                    | flags are
  FMA_SENT     = $0008;    // Sent                    | standard for
  FMA_FILE     = $0010;    // File attach             | FidoNet(tm)
  FMA_TRAN     = $0020;    // Transit                 | message attr
  FMA_ORPH     = $0040;    // Orphan                  |
  FMA_KILL     = $0080;    // Kill/sent               |
  FMA_LOCL     = $0100;    // Local                   |
  FMA_HOLD     = $0200;    // Hold                    |
  FMA_RSV2     = $0400;    // Reserved                |
  FMA_FREQ     = $0800;    // File request            |
  FMA_RREQ     = $1000;    // Return receipt request  |
  FMA_RRCP     = $2000;    // Confirmation receipt msg|
  FMA_AREQ     = $4000;    // Audit request           |
  FMA_UREQ     = $8000;    // File update request     +

  // Fts lastread file record structure

type
  _FLR = packed record (* flr *)
  	umsg: UMSG;               // Last read message uid
  end;
  FLR = _FLR;
  PFLR = ^FLR;

/////////////////////////////////////////////////////////////////////////////
// Fts api declarations
/////////////////////////////////////////////////////////////////////////////

  // Special values used by FtsReadMsg()

const
  FRM_GETCTRLCOPY =         0;     // *pcchCtrl: Allocate a copy of msg control info
  FRM_GETCTRLPTR  = ULONG(-1);     // *pcchCtrl: Return a readonly pointer to msg control info
  FRM_GETALLTEXT  = ULONG(-1);     // *pcchText: read entire message text

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FtsGetApi: PAPI; stdcall;

function FtsOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; stdcall;

function FtsCloseArea(harea: TAreaHandle): BOOL; stdcall;
function FtsLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; stdcall;

function FtsValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; stdcall;
function FtsCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; stdcall;
function FtsEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function FtsDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function FtsSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function FtsGetCurMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FtsGetFirstMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FtsGetHighMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FtsGetMsgCount(harea: TAreaHandle): LONG; stdcall;

function FtsMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; stdcall;
function FtsUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; stdcall;

function FtsOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; stdcall;
function FtsCloseMsg(hmsg: TMsgHandle): BOOL; stdcall;

function FtsLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; stdcall;
function FtsSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; stdcall;

function FtsReadMsg(hmsg: TMsgHandle; pfmh: PFMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; stdcall;

function FtsWriteMsg(hmsg: TMsgHandle; pfmh: PFMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; stdcall;

function FtsGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; stdcall;
function FtsGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; stdcall;

function FtsGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; stdcall;
function FtsSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PChar): BOOL; stdcall;
{$ENDIF}

implementation
{$IFNDEF FMA_STATIC_LINKAGE}
function FtsGetApi: PAPI; 
  external FmaDll;

function FtsOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; 
  external FmaDll;

function FtsCloseArea(harea: TAreaHandle): BOOL; 
  external FmaDll;
function FtsLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; 
  external FmaDll;

function FtsValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; 
  external FmaDll;
function FtsCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; 
  external FmaDll;
function FtsEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; 
  external FmaDll;

function FtsDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; 
  external FmaDll;
function FtsSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; 
  external FmaDll;
function FtsGetCurMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll;
function FtsGetFirstMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll;
function FtsGetHighMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll;
function FtsGetMsgCount(harea: TAreaHandle): LONG; 
  external FmaDll;

function FtsMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; 
  external FmaDll;
function FtsUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; 
  external FmaDll;

function FtsOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; 
  external FmaDll;
function FtsCloseMsg(hmsg: TMsgHandle): BOOL; 
  external FmaDll;

function FtsLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; 
  external FmaDll;
function FtsSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; 
  external FmaDll;

function FtsReadMsg(hmsg: TMsgHandle; pfmh: PFMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; 
  external FmaDll;

function FtsWriteMsg(hmsg: TMsgHandle; pfmh: PFMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; 
  external FmaDll;

function FtsGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; 
  external FmaDll;
function FtsGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; 
  external FmaDll;

function FtsGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; 
  external FmaDll;
function FtsSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PChar): BOOL; 
  external FmaDll;
{$ENDIF}

end.




