unit JamApiP;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: JAM api private declarations
//
// JamApiP.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Object Pascal by Eugene Kotlyarov 
//                                http://ekot.narod.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 13/Jan/2001
//
// History: 27/Aug/1999 kvitekp - Initially created.
//
//          10/Feb/2000 kvitekp - Added JamAddJsfFormatV()
//
//          18/Feb/2000 kvitekp - Added JamSet/CheckPassword()
//
//          13/Jan/2001 eugene  - Conversion completed.
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: STRUCTURES PACKING MUST BE THE SAME AS IN FMA!!
//
/////////////////////////////////////////////////////////////////////////////
interface
uses Windows, FmaDef, JamApi, FmaApi;

/////////////////////////////////////////////////////////////////////////////
// JAM area declarations
/////////////////////////////////////////////////////////////////////////////

  // Forward declarations
type
  PJMSG=^_JMSG;
  // JAM index cache page structure
  PJIC = ^_JIC;
  _JIC = record          (* jic *)
	pjicPrev:PJIC;                // Pointer to prev entry in list
	pjicNext:PJIC;                // Pointer to next entry in list
	cjdx	:UINT;                // Number of allocated index entries
	ijdx	:UINT;                // Index of the next unused index entry
	ajdx	:packed array [0..0] of JDX; // Index entries array
  end;
  JIC=_JIC;

  // JAM message area object structure
  PJAREA = ^ _JAREA;
  _JAREA = record        (* area *)
	magic   	:ULONG;         // Magic number (JAREA_MAGIC)
	papi		:PAPI;          // JAM api pointer
	lmhCur		:LONG;          // Current header offset
	msgnCur		:TMsgNumber;    // Current message number
	msgnBase	:TMsgNumber;    // Base message number
	msgnHigh	:TMsgNumber;    // High message number
	cmsgActive	:ULONG;         // Number of active msgs
	jbh			:JBH;           // JAM base header
	hfileJhr	:HFILE;         // JAM header file handle
	hfileJdt	:HFILE;         // JAM text file handle
	hfileJdx	:HFILE;         // JAM index file handle
	hfileJlr	:HFILE;         // JAM lastread file handle
	bAreaLocked	:BOOL;          // Area lock count
	bBaseLocked	:BOOL;          // Base lock count
	bExclusive	:BOOL;          // Exclusive count
	bJdxCached	:BOOL;          // Index caching count
	pjicList	:PJIC;          // Index cache list
	msgnLoDirty	:TMsgNumber;    // Index cache low dirty (inclusive)
	msgnHiDirty	:TMsgNumber;    // Index cache high dirty (inclusive)
	fl			:ULONG;         // Open area flags (FOA_)
	pmsgList	:PJMSG;         // Open messges list
	netAddr		:NETADDR;       // Default network address
	achPath		:packed array [0..0] of char;  // Area path (asciiz)
  end;
  JAREA=_JAREA;

  // JAM message object structure
  _JMSG = record         (* msg *)
	pmsgPrev:PJMSG;            // Pointer to prev open msg in area
	pmsgNext:PJMSG;            // Pointer to next open msg in area
	magic	:ULONG;            // Magic number (JMSG_MAGIC)
	parea	:PJAREA;           // Message area handle
	msgn	:TMsgNumber;       // Message number
	mode	:UINT;             // Message open mode
	fl		:ULONG;            // Message open flags (FOM_)
	lmh		:LONG;             // Message header offset
	jmh		:JMH;              // Message header
	pjsf	:PJSF;             // Message header subfields
	cbjsf	:ULONG;            // Message header subfields size
	cbjsfMax:ULONG;            // Message header subfields size maximum
	lmt		:LONG;             // Message text offset
	lmtEnd	:LONG;             // Message text end offset
	lmtMax	:LONG;             // Message text max offset
	fs		:UINT;             // Message status flags (JMF_)
  end;
  JMSG=_JMSG;

const 
  // JAM message area object magic number
  JAREA_MAGIC = $4552414A; // Long equivalent of "JARE"

  // JAM message object magic number
  JMSG_MAGIC  = $47534D4A;  // Long equivalent of "JMSG"

  // JAM message status flags in msg.fs

  JMF_GOTMSGH       = $0001;        // Message header has been associated
  JMF_GOTSUBF       = $0002;        // Subfield info has been associated
  JMF_SAVEJMH       = $0004;        // Save jam message header to disk
  JMF_GOTFRAME      = $0008;        // Writing frame selected for new message
  JMF_EXCLUSIVE     = $0010;        // Entered exclusive when opened
  JMF_MSGHWRITTEN   = $0100;        // Message header has been written
  JMF_SUBFWRITTEN   = $0200;        // Message contrl info has been written
  JMF_TEXTWRITTEN   = $0400;        // Message text has been written
  JMF_READDELETED   = $8000;        // Message had JMA_DELETED when last read

  // Message has been written to mask

  JMF_MASKWRITTEN   = (JMF_MSGHWRITTEN or JMF_SUBFWRITTEN or JMF_TEXTWRITTEN);

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
  function JamCheckArea(parea:PJAREA):BOOL;
    stdcall;
  function JamCheckMsg(pmsg:PJMSG):BOOL;
    stdcall;
  function JamCheckMsgReadMode(pmsg:PJMSG):BOOL;
    stdcall;
  function JamCheckMsgWriteMode(pmsg:PJMSG):BOOL;
    stdcall;

  function JamLockBase(parea:PJAREA; bLock:BOOL):BOOL;
    stdcall;
  function JamEnterExclusive(parea:PJAREA):BOOL;
    stdcall;
  function JamLeaveExclusive(parea:PJAREA):BOOL;
    stdcall;

  function JamLoadJbh(parea:PJAREA):BOOL;
    stdcall;
  function JamSaveJbh(parea:PJAREA):BOOL;
    stdcall;
  function JamCheckJbh(parea:PJAREA):BOOL;
    stdcall;

  function JamResetCurMsgCache(parea:PJAREA):BOOL;
    stdcall;
  function JamSetCurMsgCache(pmsg:PJMSG):BOOL;
    stdcall;

  function JamGetOpenMsg(parea:PJAREA; msgn:TMsgNumber):PJMSG;
    stdcall;

  function JamCheckLmh(parea:PJAREA; lmh:LONG):BOOL;
    stdcall;
  function JamLoadJmh(parea:PJAREA; pjmh:PJMH; var ppjsf:PJSF; lmh:LONG):BOOL;
    stdcall;
  function JamSaveJmh(parea:PJAREA; pjmh:PJMH; pjsf:PJSF; lmh:LONG):BOOL;
    stdcall;

  function JamGetJsf(pjsf:PJSF; cbjsf:ULONG; id:USHORT; jsfIndex:LONG):PJSF;
    stdcall;
  function JamDelJsf(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT):BOOL;
    stdcall;
  function JamAddJsf(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pb:PVOID; cb:ULONG):PJSF;
    stdcall;
//  function _JamAddJsfFormat(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pszFormat:PSZ{, ...}):PJSF;
//	cdecl;
  function JamAddJsfFormatV(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pszFormat:PSZ; parg:va_list):PJSF;
    stdcall;

  function JamBeginJdxCache(parea:PJAREA):BOOL;
    stdcall;
  function JamEndJdxCache(parea:PJAREA):BOOL;
    stdcall;

  function JamGetJdxCount(parea:PJAREA):ULONG;
    stdcall;

  function JamGetJdx(parea:PJAREA; pjdx:PJDX; msgn:TMsgNumber):BOOL;
    stdcall;
  function JamSetJdx(parea:PJAREA; pjdx:PJDX; msgn:TmsgNumber):BOOL;
    stdcall;
  function JamDelJdx(parea:PJAREA; pjdx:PJDX; msgn:TMsgNumber):BOOL;
    stdcall;
  function JamAddJdx(parea:PJAREA; pjdx:PJDX; var pmsgn:TMsgNumber):BOOL;
    stdcall;

  function JamGetCrc32(pch:PCH; cch:ULONG; crc:ULONG):ULONG;
    stdcall;

  function JamGetJlr(parea:PJAREA; uid:ULONG; pjlr:PJLR; var poffs:LONG):BOOL;
    stdcall;
  function JamSetJlr(parea:PJAREA; pjlr:PJLR):BOOL;
    stdcall;

  function JamCheckBase(pszPath:PSZ):BOOL;
    stdcall;

  function JamSetPassword(parea:PJAREA; pszPassword:PSZ):BOOL;
    stdcall;
  function JamCheckPassword(parea:PJAREA; pszPassword:PSZ):BOOL;
    stdcall;
{$ENDIF}
implementation
{$IFNDEF FMA_STATIC_LINKAGE}
  function JamCheckArea(parea:PJAREA):BOOL;
    external FmaDll;
  function JamCheckMsg(pmsg:PJMSG):BOOL;
    external FmaDll;
  function JamCheckMsgReadMode(pmsg:PJMSG):BOOL;
    external FmaDll;
  function JamCheckMsgWriteMode(pmsg:PJMSG):BOOL;
    external FmaDll;

  function JamLockBase(parea:PJAREA; bLock:BOOL):BOOL;
    external FmaDll;
  function JamEnterExclusive(parea:PJAREA):BOOL;
    external FmaDll;
  function JamLeaveExclusive(parea:PJAREA):BOOL;
    external FmaDll;

  function JamLoadJbh(parea:PJAREA):BOOL;
    external FmaDll;
  function JamSaveJbh(parea:PJAREA):BOOL;
    external FmaDll;
  function JamCheckJbh(parea:PJAREA):BOOL;
    external FmaDll;

  function JamResetCurMsgCache(parea:PJAREA):BOOL;
    external FmaDll;
  function JamSetCurMsgCache(pmsg:PJMSG):BOOL;
    external FmaDll;

  function JamGetOpenMsg(parea:PJAREA; msgn:TMsgNumber):PJMSG;
    external FmaDll;

  function JamCheckLmh(parea:PJAREA; lmh:LONG):BOOL;
    external FmaDll;
  function JamLoadJmh(parea:PJAREA; pjmh:PJMH; var ppjsf:PJSF; lmh:LONG):BOOL;
    external FmaDll;
  function JamSaveJmh(parea:PJAREA; pjmh:PJMH; pjsf:PJSF; lmh:LONG):BOOL;
    external FmaDll;

  function JamGetJsf(pjsf:PJSF; cbjsf:ULONG; id:USHORT; jsfIndex:LONG):PJSF;
    external FmaDll;
  function JamDelJsf(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT):BOOL;
    external FmaDll;
  function JamAddJsf(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pb:PVOID; cb:ULONG):PJSF;
    external FmaDll;
//  function _JamAddJsfFormat(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pszFormat:PSZ{, ...}):PJSF;
//  	external FmaDll;
  function JamAddJsfFormatV(var ppjsf:PJSF; var pcbjsf:ULONG; id:USHORT; pszFormat:PSZ; parg:va_list):PJSF;
    external FmaDll;

  function JamBeginJdxCache(parea:PJAREA):BOOL;
    external FmaDll;
  function JamEndJdxCache(parea:PJAREA):BOOL;
    external FmaDll;

  function JamGetJdxCount(parea:PJAREA):ULONG;
    external FmaDll;

  function JamGetJdx(parea:PJAREA; pjdx:PJDX; msgn:TMsgNumber):BOOL;
    external FmaDll;
  function JamSetJdx(parea:PJAREA; pjdx:PJDX; msgn:TmsgNumber):BOOL;
    external FmaDll;
  function JamDelJdx(parea:PJAREA; pjdx:PJDX; msgn:TMsgNumber):BOOL;
    external FmaDll;
  function JamAddJdx(parea:PJAREA; pjdx:PJDX; var pmsgn:TMsgNumber):BOOL;
    external FmaDll;

  function JamGetCrc32(pch:PCH; cch:ULONG; crc:ULONG):ULONG;
    external FmaDll;

  function JamGetJlr(parea:PJAREA; uid:ULONG; pjlr:PJLR; var poffs:LONG):BOOL;
    external FmaDll;
  function JamSetJlr(parea:PJAREA; pjlr:PJLR):BOOL;
    external FmaDll;

  function JamCheckBase(pszPath:PSZ):BOOL;
    external FmaDll;

  function JamSetPassword(parea:PJAREA; pszPassword:PSZ):BOOL;
    external FmaDll;
  function JamCheckPassword(parea:PJAREA; pszPassword:PSZ):BOOL;
    external FmaDll;
{$ENDIF}
end.