unit PktApiP;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Pkt api private declarations module
//
// PktApiP.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Object Pascal by Eugene Kotlyarov 
//                                http://ekot.narod.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 08/Feb/2000
// Updated: 13/Jan/2001
//
// History: 08/Feb/2000 kvitekp - Initially created.
//
//          18/Feb/2000 kvitekp - Added PktSet/CheckPassword()
//
//          13/Jan/2001 eugene  - Conversion completed.
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: STRUCTURES PACKING MUST BE THE SAME AS IN FMA!!
//
/////////////////////////////////////////////////////////////////////////////
interface
uses Windows, FtsApi, PktApi, FmaApi, FmaDef;

/////////////////////////////////////////////////////////////////////////////
// Pkt area declarations
/////////////////////////////////////////////////////////////////////////////

  // Forward declarations

type
  PPMSG=^_PMSG;

  // Packed message info structure
  _PMI = record                     (* pmi *)                           
    lmh  :LONG;                         // Message header offset      
    cbHdr:ULONG;                        // Message header length      
    cbMsg:ULONG;                        // Message length (hdr + body)
    umsg :ULONG;                        // Message unique id          
  end;

  PPMI=^_PMI;
  // Pkt message area object structure

  _PAREA = record        (* area *)
	magic		:ULONG;             // Magic number (PAREA_MAGIC)
    papi		:PAPI;              // Pkt api pointer
	pbh    		:PBH;               // Packet header
	hfilePkt	:HFILE;             // Packet file handle
	bAreaLocked	:BOOL;              // Area lock count
	bBaseLocked	:BOOL;              // Base lock count
	bExclusive	:BOOL;              // Exclusive count
	cmsg		:ULONG;             // Number of messages
	msgnCur		:TMsgNumber;		// Current message number
	msgnBase	:TMsgNumber;        // Base message number
	msgnHigh	:TMsgNumber;        // Highest message number
	apmi		:PPMI;              // Packed message info array
	cpmi		:UINT;              // Packed message info array size
	fl			:ULONG;             // Open area flags (FOA_)
	pmsgList	:PPMSG;            	// Open messges list
	netAddr		:NETADDR;           // Default network address
	netOrig		:NETADDR;           // Packet origination network address
	netDest		:NETADDR;           // Packet destination network address
	dttm		:DTTM;              // Packet creation date and time
	achPath		:packed array[0..0] of CHAR; // Area path (asciiz)
  end;	
  PPAREA=^_PAREA;
  PAREA=_PAREA;

  // Pkt message object structure
  _PMSG = record         (* msg *)
	pmsgPrev	:PPMSG;            // Pointer to prev open msg in area
	pmsgNext	:PPMSG;            // Pointer to next open msg in area
	magic		:ULONG;            // Magic number (PMSG_MAGIC)
	parea		:PPAREA;           // Message area handle
	msgn		:TMsgNumber;       // Message number
	mode		:UINT;             // Message open mode
	fl			:ULONG;            // Message open flags (FOM_)
	pszCtrl		:PSZ;              // Message control info
	cchCtrl		:UINT;             // Message control info length
	fmh			:FMH;              // Message header
	lmh			:LONG;             // Message header offset
	cbHdr		:ULONG;            // Message header length
	cbMsg		:ULONG;            // Message length (hdr + body)
	lmt			:LONG;             // Message text offset
	lmtEnd		:LONG;             // Message text end offset after writing
	fs			:UINT;             // Message status flags (PMF_)
  end;              // Note: PMSG conflicts with Windows.h

  // Pkt message area object magic number
const PAREA_MAGIC =$45524150; // Long equivalent of "PARE"

  // Pkt message object magic number
const PMSG_MAGIC = $47534D50;  // Long equivalent of "PMSG"

  // Pkt message status flags in msg.fs

const PMF_GOTMSGH     =  $0001;        // Message header has been associated
const PMF_GOTCTRL     =  $0002;        // Control info has been associated
const PMF_BOGUS       =  $0004;        // -- reserved
const PMF_GOTFRAME    =  $0008;        // Writing frame selected for new message
const PMF_EXCLUSIVE   =  $0010;        // Entered exclusive when opened
const PMF_MSGHWRITTEN =  $0100;        // Message header has been written
const PMF_CTRLWRITTEN =  $0200;        // Message contrl info has been written
const PMF_TEXTWRITTEN =  $0400;        // Message text has been written
const PMF_CTRLHASAREA =  $1000;        // Message contrl info has AREA w/o ^a

  // Message has been written to mask

const PMF_MASKWRITTEN =  (PMF_MSGHWRITTEN or PMF_CTRLWRITTEN or PMF_TEXTWRITTEN);

{$IFNDEF FMA_STATIC_LINKAGE}
function PktCheckArea(parea:PPAREA):BOOL;
	stdcall;
function PktCheckMsg(pmsg:PPMSG):BOOL;
	stdcall;
function PktCheckMsgReadMode(pmsg:PPMSG):BOOL;
	stdcall;
function PktCheckMsgWriteMode(pmsg:PPMSG):BOOL;
	stdcall;

function PktLockBase(parea:PPAREA; bLock:BOOL):BOOL;
	stdcall;
function PktEnterExclusive(parea:PPAREA):BOOL;
	stdcall;
function PktLeaveExclusive(parea:PPAREA):BOOL;
	stdcall;
function PktSetEnd(parea:PPAREA; offs:LONG):BOOL;
	stdcall;
function PktRescanArea(parea:PPAREA):BOOL;
	stdcall;

function PktLoadPbh(parea:PPAREA):BOOL;
	stdcall;
function PktSavePbh(parea:PPAREA):BOOL;
	stdcall;
function PktCheckPbh(parea:PPAREA):BOOL;
	stdcall;

function PktResetCurMsgCache(parea:PPAREA):BOOL;
	stdcall;
function PktSetCurMsgCache(pmsg:PPMSG):BOOL;
	stdcall;

function PktGetOpenMsg(parea:PPAREA; msgn:TMsgNumber):PPMSG;
    stdcall;
function PktLoadPmh(parea:PPAREA; pfmh:PFMH; lmh:LONG):BOOL;
	stdcall;
function PktSavePmh(parea:PPAREA; pfmh:PFMH; lmh:LONG):BOOL;
	stdcall;

function PktAddMsg(parea:PPAREA; lmh:LONG; cbHdr,cbMsg:ULONG):BOOL;
	stdcall;
function PktDelMsg(parea:PPAREA; imsg:UINT):BOOL;
	stdcall;
function PktShrinkPkt(parea:PPAREA; offs:LONG; cb:ULONG):BOOL;
	stdcall;
function PktExpandPkt(parea:PPAREA; offs:LONG; cb:ULONG):BOOL;
	stdcall;

function PktCheckBase(pszPath:PSZ):BOOL;
	stdcall;

function PktSetNetAddr(parea:PPAREA; var pnetOrig, pnetDest:NETADDR):BOOL;
	stdcall;
function PktGetNetAddr(parea:PPAREA; var pnetOrig, pnetDest:NETADDR):BOOL;
	stdcall;

function PktSetPassword(parea:PPAREA; pszPassword:PSZ):BOOL;
	stdcall;
function PktCheckPassword(parea:PPAREA; pszPassword:PSZ):BOOL;
	stdcall;

{$ENDIF}
implementation

{$IFNDEF FMA_STATIC_LINKAGE}
function PktCheckArea(parea:PPAREA):BOOL;
	external FmaDll name 'PktCheckArea';
function PktCheckMsg(pmsg:PPMSG):BOOL;
	external FmaDll name 'PktCheckMsg';
function PktCheckMsgReadMode(pmsg:PPMSG):BOOL;
	external FmaDll name 'PktCheckMsgReadMode';
function PktCheckMsgWriteMode(pmsg:PPMSG):BOOL;
	external FmaDll name 'PktCheckMsgWriteMode';

function PktLockBase(parea:PPAREA; bLock:BOOL):BOOL;
	external FmaDll name 'PktLockBase';
function PktEnterExclusive(parea:PPAREA):BOOL;
	external FmaDll name 'PktEnterExclusive';
function PktLeaveExclusive(parea:PPAREA):BOOL;
	external FmaDll name 'PktLeaveExclusive';
function PktSetEnd(parea:PPAREA; offs:LONG):BOOL;
	external FmaDll name 'PktSetEnd';
function PktRescanArea(parea:PPAREA):BOOL;
	external FmaDll name 'PktRescanArea';

function PktLoadPbh(parea:PPAREA):BOOL;
	external FmaDll name 'PktLoadPbh';
function PktSavePbh(parea:PPAREA):BOOL;
	external FmaDll name 'PktSavePbh';
function PktCheckPbh(parea:PPAREA):BOOL;
	external FmaDll name 'PktCheckPbh';

function PktResetCurMsgCache(parea:PPAREA):BOOL;
	external FmaDll name 'PktResetCurMsgCache';
function PktSetCurMsgCache(pmsg:PPMSG):BOOL;
	external FmaDll name 'PktSetCurMsgCache';

function PktGetOpenMsg(parea:PPAREA; msgn:TMsgNumber):PPMSG;
    external FmaDll name 'PktGetCurOpenmsg';
function PktLoadPmh(parea:PPAREA; pfmh:PFMH; lmh:LONG):BOOL;
	external FmaDll name 'PktLoadPmh';
function PktSavePmh(parea:PPAREA; pfmh:PFMH; lmh:LONG):BOOL;
	external FmaDll name 'PktSavePmh';

function PktAddMsg(parea:PPAREA; lmh:LONG; cbHdr,cbMsg:ULONG):BOOL;
	external FmaDll name 'PktAddMsg';
function PktDelMsg(parea:PPAREA; imsg:UINT):BOOL;
	external FmaDll name 'PktDelMsg';
function PktShrinkPkt(parea:PPAREA; offs:LONG; cb:ULONG):BOOL;
	external FmaDll name 'PktShrinkPkt';
function PktExpandPkt(parea:PPAREA; offs:LONG; cb:ULONG):BOOL;
	external FmaDll name 'PktExpandPkt';

function PktCheckBase(pszPath:PSZ):BOOL;
	external FmaDll name 'PktCheckBase';

function PktSetNetAddr(parea:PPAREA; var pnetOrig, pnetDest:NETADDR):BOOL;
	external FmaDll name 'PktSetNetAddr';
function PktGetNetAddr(parea:PPAREA; var pnetOrig, pnetDest:NETADDR):BOOL;
	external FmaDll name 'PktGetNetAddr';

function PktSetPassword(parea:PPAREA; pszPassword:PSZ):BOOL;
	external FmaDll name 'PktSetPassword';
function PktCheckPassword(parea:PPAREA; pszPassword:PSZ):BOOL;
	external FmaDll name 'PktCheckPassword';

{$ENDIF}
end.
