unit SquApiP;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Squish api private declarations
//
// SquApiP.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Object Pascal by Eugene Kotlyarov 
//                                http://ekot.narod.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 13/Jan/2001
//
// History: 27/Aug/1999 kvitekp - Initially created.
//
//          13/Jan/2001 eugene  - Conversion completed.
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: STRUCTURES PACKING MUST BE THE SAME AS IN FMA!!
//
/////////////////////////////////////////////////////////////////////////////
interface
uses Windows, FmaDef, SquApi, FmaApi;

/////////////////////////////////////////////////////////////////////////////
// Squish area declarations
/////////////////////////////////////////////////////////////////////////////
type
  // Forward declarations

  PSMSG = ^_SMSG;

  // Squish index cache page structure
  PSIC = ^SIC;
  _SIC = record           (* sic *)
    psicPrev:PSIC;             // Pointer to prev entry in list
    psicNext:PSIC;             // Pointer to next entry in list
    csqi	:UINT;                       // Number of allocated index entries
    isqi	:UINT;                       // Index of the next unused index entry
    asqi	: packed array [0..0] of SQI;                    // Index entries array
  end;
  SIC = _SIC;

  // Squish message area object structure
  _SAREA = record         (* area *)
    magic		:ULONG;                // Magic number (SAREA_MAGIC)
    papi		:PAPI;                 // Squish api pointer
    sbh			:SBH;                  // Squish base header
    hfileSqd	:HFILE;                // Squish data file handle
    hfileSqi	:HFILE;                // Squish index file handle
    hfileSql	:HFILE;                // Squish lastread file handle
    bAreaLocked	:BOOL;                 // Area lock count
    bBaseLocked	:BOOL;                 // Base lock count
    bExclusive	:BOOL;                 // Exclusive count
    bSqiCached	:BOOL;                 // Index caching count
    psicList	:PSIC;                 // Index cache list
    msgnLoDirty	:TMsgNumber;           // Index cache low dirty (inclusive)
    msgnHiDirty	:TMsgNumber;                 // Index cache high dirty (inclusive)
    msgnCur		:TMsgNumber;                 // Current message number
    lfmCur		:LONG;                 // Current message frame offset
    lfmPrev		:LONG;                 // Prev message frame offset
    lfmNext		:LONG;                 // Next message frame offset
    fl			:ULONG;                // Open area flags (FOA_)
    pmsgList	:PSMSG;                // Open messges list
    netAddr		:NETADDR;              // Default network address
    achPath		: packed array [0..0] of CHAR; // Area path (asciiz)
  end;
  SAREA = _SAREA;
  PSAREA = ^_SAREA;

  // Squish message object structure
  _SMSG = record         (* msg *)
    pmsgPrev:PSMSG;            // Pointer to prev open msg in area
    pmsgNext:PSMSG;            // Pointer to next open msg in area
    magic	:ULONG;            // Magic number (SMSG_MAGIC)
    parea	:PSAREA;           // Message area handle
    msgn	:TMsgNumber;       // Message number
    mode	:UINT;             // Message open mode
    fl		:ULONG;            // Message open flags (FOM_)
    pszCtrl	:PSZ;              // Message control info
    cchCtrl	:UINT;             // Message control info length
    smh		:SMH;              // Message header
    sfh		:SFH;              // Message frame header
    fs		:UINT;             // Message status flags (SMF_)
    lfm		:LONG;             // Message frame offset
    umsg	:UMSG;             // Message unique id
  end;
  SMSG = _SMSG;

const

  // Squish message area object magic number
 SAREA_MAGIC = $45524153; // Long equivalent of "SARE"

  // Squish message object magic number
 SMSG_MAGIC = $47534D53;  // Long equivalent of "SMSG"

  // Squish message status flags in msg.fs
 SMF_GOTMSGH       =$0001;        // Message header has been associated
 SMF_GOTCTRL       =$0002;        // Control info has been associated
 SMF_SAVESFH       =$0004;        // Save squish frame header to disk
 SMF_GOTFRAME      =$0008;        // Writing frame selected for new message
 SMF_EXCLUSIVE     =$0010;        // Entered exclusive when opened
 SMF_MSGHWRITTEN   =$0100;        // Message header has been written
 SMF_CTRLWRITTEN   =$0200;        // Message contrl info has been written
 SMF_TEXTWRITTEN   =$0400;        // Message text has been written

  // Message has been written to mask
 SMF_MASKWRITTEN   = (SMF_MSGHWRITTEN or SMF_CTRLWRITTEN or SMF_TEXTWRITTEN);

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function SquCheckArea(parea:PSAREA):BOOL; stdcall;
function SquCheckMsg(pmsg:PSMSG):BOOL; stdcall;
function SquCheckMsgReadMode(pmsg:PSMSG):BOOL; stdcall;
function SquCheckMsgWriteMode(pmsg:PSMSG):BOOL; stdcall;

function SquLockBase(parea:PSAREA; bLock:BOOL):BOOL; stdcall;
function SquEnterExclusive(parea:PSAREA):BOOL; stdcall;
function SquLeaveExclusive(parea:PSAREA):BOOL; stdcall;

function SquLoadSbh(parea:PSAREA):BOOL; stdcall;
function SquSaveSbh(parea:PSAREA):BOOL; stdcall;
function SquCheckSbh(parea:PSAREA):BOOL; stdcall;

function SquResetCurMsgCache(parea:PSAREA):BOOL; stdcall;
function SquSetCurMsgCache(pmsg:PSMSG):BOOL; stdcall;

function SquLoadSfh(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;
function SquSaveSfh(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;

function SquAddFrame(parea:PSAREA; plfm:PLONG; cbFrame:ULONG):BOOL; stdcall;
function SquGetFrame(parea:PSAREA; plfm:PLONG; msgn:TMsgNumber):BOOL; stdcall;
function SquSetNextFrame(parea:PSAREA; lfm:LONG; lfmNext:LONG):BOOL; stdcall;
function SquSetPrevFrame(parea:PSAREA; lfm:LONG; lfmPrev:LONG):BOOL; stdcall;

function SquCheckUsed(parea:PSAREA; lfm:LONG):BOOL; stdcall;
function SquAddUsed(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;
function SquDelUsed(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;

function SquCheckFree(parea:PSAREA; lfm:LONG):BOOL; stdcall;
function SquAddFree(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;
function SquDelFree(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; stdcall;
function SquFindFree(parea:PSAREA; psfh:PSFH; plfm:PLONG; cbFrame:ULONG):BOOL; stdcall;

function SquGetOpenMsg(parea:PSAREA; msgn:TMsgNumber):PSMSG; stdcall;
function SquDelMsg(parea:PSAREA; msgn:TMsgNumber; psfh:PSFH; lfm:LONG):BOOL; stdcall;
function SquTrimBase(parea:PSAREA; bAdding:BOOL):BOOL; stdcall;

function SquLoadSmh(parea:PSAREA; psmh:PSMH; lfm:LONG):BOOL; stdcall;
function SquSaveSmh(parea:PSAREA; psmh:PSMH; lfm:LONG):BOOL; stdcall;
function SquGetSafeMsgCtrlLen(pmsg:PSMSG; pcbMsg,pcbCtrl:PULONG):BOOL; stdcall;

function SquBeginSqiCache(parea:PSAREA):BOOL; stdcall;
function SquEndSqiCache(parea:PSAREA):BOOL; stdcall;

function SquGetSqiCount(parea:PSAREA):ULONG; stdcall;

function SquGetSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; stdcall;
function SquSetSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; stdcall;
function SquDelSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; stdcall;
function SquAddSqi(parea:PSAREA; psqi:PSQI; var pmsgn:TMsgNumber):BOOL; stdcall;
function SquAppSqi(parea:PSAREA; psqi:PSQI; var pmsgn:TMsgnumber):BOOL; stdcall;

function SquSetBaseTrimming(parea:PSAREA; cmsgMax, cmsgSkip:ULONG; 
										 keepDays:USHORT):BOOL; stdcall;

function SquGetHash(pchText:PCH; cchText:UINT):ULONG; stdcall;

function SquCheckBase(pszPath:PSZ):BOOL; stdcall;
{$ENDIF}
implementation
{$IFNDEF FMA_STATIC_LINKAGE}
function SquCheckArea(parea:PSAREA):BOOL; external FmaDll;
function SquCheckMsg(pmsg:PSMSG):BOOL; external FmaDll;
function SquCheckMsgReadMode(pmsg:PSMSG):BOOL; external FmaDll;
function SquCheckMsgWriteMode(pmsg:PSMSG):BOOL; external FmaDll;

function SquLockBase(parea:PSAREA; bLock:BOOL):BOOL; external FmaDll;
function SquEnterExclusive(parea:PSAREA):BOOL; external FmaDll;
function SquLeaveExclusive(parea:PSAREA):BOOL; external FmaDll;

function SquLoadSbh(parea:PSAREA):BOOL; external FmaDll;
function SquSaveSbh(parea:PSAREA):BOOL; external FmaDll;
function SquCheckSbh(parea:PSAREA):BOOL; external FmaDll;

function SquResetCurMsgCache(parea:PSAREA):BOOL; external FmaDll;
function SquSetCurMsgCache(pmsg:PSMSG):BOOL; external FmaDll;

function SquLoadSfh(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;
function SquSaveSfh(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;

function SquAddFrame(parea:PSAREA; plfm:PLONG; cbFrame:ULONG):BOOL; external FmaDll;
function SquGetFrame(parea:PSAREA; plfm:PLONG; msgn:TMsgNumber):BOOL; external FmaDll;
function SquSetNextFrame(parea:PSAREA; lfm:LONG; lfmNext:LONG):BOOL; external FmaDll;
function SquSetPrevFrame(parea:PSAREA; lfm:LONG; lfmPrev:LONG):BOOL; external FmaDll;

function SquCheckUsed(parea:PSAREA; lfm:LONG):BOOL; external FmaDll;
function SquAddUsed(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;
function SquDelUsed(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;

function SquCheckFree(parea:PSAREA; lfm:LONG):BOOL; external FmaDll;
function SquAddFree(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;
function SquDelFree(parea:PSAREA; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;
function SquFindFree(parea:PSAREA; psfh:PSFH; plfm:PLONG; cbFrame:ULONG):BOOL; external FmaDll;

function SquGetOpenMsg(parea:PSAREA; msgn:TMsgNumber):PSMSG; external FmaDll;
function SquDelMsg(parea:PSAREA; msgn:TMsgNumber; psfh:PSFH; lfm:LONG):BOOL; external FmaDll;
function SquTrimBase(parea:PSAREA; bAdding:BOOL):BOOL; external FmaDll;

function SquLoadSmh(parea:PSAREA; psmh:PSMH; lfm:LONG):BOOL; external FmaDll;
function SquSaveSmh(parea:PSAREA; psmh:PSMH; lfm:LONG):BOOL; external FmaDll;
function SquGetSafeMsgCtrlLen(pmsg:PSMSG; pcbMsg,pcbCtrl:PULONG):BOOL; external FmaDll;

function SquBeginSqiCache(parea:PSAREA):BOOL; external FmaDll;
function SquEndSqiCache(parea:PSAREA):BOOL; external FmaDll;

function SquGetSqiCount(parea:PSAREA):ULONG; external FmaDll;

function SquGetSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; external FmaDll;
function SquSetSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; external FmaDll;
function SquDelSqi(parea:PSAREA; psqi:PSQI; msgn:TMsgNumber):BOOL; external FmaDll;
function SquAddSqi(parea:PSAREA; psqi:PSQI; var pmsgn:TMsgNumber):BOOL; external FmaDll;
function SquAppSqi(parea:PSAREA; psqi:PSQI; var pmsgn:TMsgnumber):BOOL; external FmaDll;

function SquSetBaseTrimming(parea:PSAREA; cmsgMax, cmsgSkip:ULONG; 
										 keepDays:USHORT):BOOL; external FmaDll;

function SquGetHash(pchText:PCH; cchText:UINT):ULONG; external FmaDll;

function SquCheckBase(pszPath:PSZ):BOOL; external FmaDll;
{$ENDIF}
end.